from dbt.adapters.sql import SQLAdapter
from dbt.adapters.mindsdb import MindsdbConnectionManager


LIST_SCHEMAS_MACRO_NAME = 'list_schemas'
LIST_RELATIONS_MACRO_NAME = 'list_relations_without_caching'

class MindsdbAdapter(SQLAdapter):
    ConnectionManager = MindsdbConnectionManager

    @classmethod
    def date_function(cls):
        return 'current_date()'

    def quote(self, identifier):
        return '`{}`'.format(identifier)



from dataclasses import dataclass
from contextlib import contextmanager
import dbt.exceptions
from dbt.contracts.connection import AdapterResponse

from dbt.adapters.base import Credentials
from dbt.adapters.sql import SQLConnectionManager
from dbt.logger import GLOBAL_LOGGER as logger
from typing import Optional
import dbt.exceptions
import mysql.connector

@dataclass
class MindsdbCredentials(Credentials):
    host: str
    port: int
    username: str
    password: str
    database: str
    schema: str

    @property
    def type(self):
        return 'mindsdb'

    def _connection_keys(self):
        """
        List of keys to display in the `dbt debug` output.
        """
        return 'host', 'port', 'user', 'database', 'schema'



class MindsdbConnectionManager(SQLConnectionManager):
    TYPE = 'mindsdb'

    @classmethod
    def get_response(cls, cursor) -> AdapterResponse:
        return AdapterResponse(
            _message="{}".format(f"OK. Rows affected: {cursor.rowcount}"),
            rows_affected=cursor.rowcount,
            code=200
        )
        
    @classmethod
    def open(cls, connection):
        if connection.state == 'open':
            logger.debug('Connection is already open, skipping open.')
            return connection

        credentials = connection.credentials

        try:
            handle = mysql.connector.connect(
                host=credentials.host,
                port=credentials.port,
                username=credentials.username,
                password=credentials.password,
                database=credentials.database
            )
            connection.state = 'open'
            connection.handle = handle
        except mysql.connector.Error as exc:
            logger.debug(
                "Got an error when attempting to open a "
                "connection: '{}'".format(exc)
            )
            connection.handle = None
            connection.state = "fail"

            raise dbt.exceptions.FailedToConnectException(str(exc))
        return connection


    #TODO: Remove this if mindsdb sql api doesn't support canceling ongoing queries   
    def cancel(self, connection):
        pass
    
    @contextmanager
    def exception_handler(self, sql: str):
        try:
            yield
        except mysql.connector.DatabaseError as exc:
            logger.debug('myadapter error: {}'.format(str(exc)))
            raise dbt.exceptions.DatabaseException(str(exc))
        except Exception as exc:
            logger.debug("Error running SQL: {}".format(sql))
            logger.debug("Rolling back transaction.")
            raise dbt.exceptions.RuntimeException(str(exc))




from dbt.adapters.mindsdb.connections import MindsdbConnectionManager
from dbt.adapters.mindsdb.connections import MindsdbCredentials
from dbt.adapters.mindsdb.impl import MindsdbAdapter

from dbt.adapters.base import AdapterPlugin
from dbt.include import mindsdb


Plugin = AdapterPlugin(
    adapter=MindsdbAdapter,
    credentials=MindsdbCredentials,
    include_path=mindsdb.PACKAGE_PATH)



#!/usr/bin/env python
from setuptools import find_namespace_packages, setup

package_name = "dbt-mindsdb"
# make sure this always matches dbt/adapters/mindsdb/__version__.py
package_version = "1.0.1"
description = """The dbt adapter plugin for connecting to MindsDB"""

setup(
    name=package_name,
    version=package_version,
    description=description,
    long_description=description,
    author='MindsDB Inc',
    author_email='admin@mindsdb.com',
    url='https://github.com/mindsdb/dbt-mindsdb',
    packages=find_namespace_packages(include=['dbt', 'dbt.*']),
    include_package_data=True,
    install_requires=[
        "dbt-core==1.0.1",
        "mysql-connector-python~=8.0.22",
    ]
)
